@@ -3,18 +3,26 @@ require_relative '../core_ext/hash'
 
 require_relative 'host'
 
+require_relative 'color'
 require_relative 'command'
 require_relative 'command_map'
 require_relative 'configuration'
 require_relative 'coordinator'
 
+require_relative 'deprecation_logger'
+require_relative 'dsl'
+
+require_relative 'exception'
+
 require_relative 'logger'
 require_relative 'log_message'
 
+require_relative 'mapping_interaction_handler'
+
 require_relative 'formatters/abstract'
 require_relative 'formatters/black_hole'
-require_relative 'formatters/simple_text'
 require_relative 'formatters/pretty'
+require_relative 'formatters/simple_text'
 require_relative 'formatters/dot'
 
 require_relative 'runners/abstract'
@@ -27,7 +35,6 @@ require_relative 'backends/abstract'
 require_relative 'backends/connection_pool'
 require_relative 'backends/printer'
 require_relative 'backends/netssh'
+require_relative 'backends/netssh/known_hosts'
 require_relative 'backends/local'
 require_relative 'backends/skipper'
-
-require_relative 'utils/capture_output_methods'
