@@ -21,25 +21,15 @@ module SSHKit
     end
 
     def initialize(host_string_or_options_hash)
+      @keys  = []
+      @local = false
 
-      unless host_string_or_options_hash.is_a?(Hash)
-        suitable_parsers = [
-          SimpleHostParser,
-          HostWithPortParser,
-          HostWithUsernameAndPortParser,
-          IPv6HostWithPortParser,
-          HostWithUsernameParser,
-        ].select do |p|
-          p.suitable?(host_string_or_options_hash)
-        end
-
-        if suitable_parsers.any?
-          suitable_parsers.first.tap do |parser|
-            @user, @hostname, @port = parser.new(host_string_or_options_hash).attributes
-          end
-        else
-          raise UnparsableHostStringError, "Cannot parse host string #{host_string_or_options_hash}"
-        end
+      if host_string_or_options_hash == :local
+        @local = true
+        @hostname = "localhost"
+        @user = ENV['USER'] || ENV['LOGNAME'] || ENV['USERNAME']
+      elsif !host_string_or_options_hash.is_a?(Hash)
+        @user, @hostname, @port = first_suitable_parser(host_string_or_options_hash).attributes
       else
         host_string_or_options_hash.each do |key, value|
           if self.respond_to?("#{key}=")
@@ -51,6 +41,10 @@ module SSHKit
       end
     end
 
+    def local?
+      @local
+    end
+
     def hash
       user.hash ^ hostname.hash ^ port.hash
     end
@@ -84,6 +78,11 @@ module SSHKit
       @properties ||= OpenStruct.new
     end
 
+    def first_suitable_parser(host)
+      parser = PARSERS.find{|p| p.suitable?(host) }
+      fail UnparsableHostStringError, "Cannot parse host string #{host}" if parser.nil?
+      parser.new(host)
+    end
   end
 
   # @private
@@ -91,7 +90,7 @@ module SSHKit
   class SimpleHostParser
 
     def self.suitable?(host_string)
-      !host_string.match /[:|@]/
+      !host_string.match(/:|@/)
     end
 
     def initialize(host_string)
@@ -119,7 +118,7 @@ module SSHKit
   class HostWithPortParser < SimpleHostParser
 
     def self.suitable?(host_string)
-      !host_string.match /[@|\[|\]]/
+      !host_string.match(/@|\[|\]/)
     end
 
     def port
@@ -136,7 +135,7 @@ module SSHKit
   # :nodoc:
   class HostWithUsernameAndPortParser < SimpleHostParser
     def self.suitable?(host_string)
-      host_string.match /@.*:\d+/
+      host_string.match(/@.*:\d+/)
     end
     def username
       @host_string.split(/:|@/)[0]
@@ -154,7 +153,7 @@ module SSHKit
   class IPv6HostWithPortParser < SimpleHostParser
 
     def self.suitable?(host_string)
-      host_string.match /[a-fA-F0-9:]+:\d+/
+      host_string.match(/[a-fA-F0-9:]+:\d+/)
     end
 
     def port
@@ -182,4 +181,12 @@ module SSHKit
     end
   end
 
+  PARSERS = [
+    SimpleHostParser,
+    HostWithPortParser,
+    HostWithUsernameAndPortParser,
+    IPv6HostWithPortParser,
+    HostWithUsernameParser,
+  ].freeze
+
 end
